(*:maxLineLen=78:*)

theory LRI_2016_Slides
  imports Pure
begin

section \<open>Abstract\<close>

text \<open>\tiny
  Isabelle is usually positioned as environment for interactive and automated
  theorem proving, but its Prover IDE (PIDE) may be used for regular program
  development as well. Standard ML is particularly important here, since it is
  the bootstrap language of Isabelle/ML (i.e. SML with many add-ons) and
  Isabelle/Pure (i.e. the logical framework).

  The ML IDE functionality of Isabelle + Poly/ML is manifold:

  \<^item> Continuous feedback from static analysis and semantic evaluation is already
  available for years, e.g. Isabelle2014 (August 2014). It is a corollary of how
  PIDE interaction works, and of the integration of the Poly/ML compiler into
  that framework. Source files are statically checked and semantically evaluated
  while the user is editing. The annotated sources contain markup about inferred
  types, references to defining positions of items etc.

  \<^item> Source-level debugging within the IDE is new in Poly/ML 5.6, which is
  bundled with Isabelle2016 (February 2016). The Prover IDE provides the
  Debugger dockable to connect to running ML threads, inspect the stack frame
  with local ML bindings, and evaluate ML expressions in a particular run-time
  context. See also
  \<^url>\<open>http://sketis.net/2016/ml-debugging-within-the-prover-ide\<close>.

  \<^item> IDE support for the Isabelle/Pure bootstrap process is new technology for
  the coming release of Isabelle2016-1 (December 2016). The \<^verbatim>\<open>ROOT.ML\<close> file acts
  like a quasi-theory in the context of theory \<^verbatim>\<open>ML_Bootstrap\<close>: this allows
  continuous checking of all loaded ML files. The theory file is presented with
  a modified header to import Pure from the running Isabelle instance.

  It is also possible to modify standalone SML projects, to edit the sources
  freely in the ML IDE. For example, MetiTarski
  \<^url>\<open>https://bitbucket.org/lcpaulson/metitarski\<close> can participate after some
  trivial changes of its \<^verbatim>\<open>ROOT.ML\<close> file.

  Overall, we move more and more to an integrated framework for development of
  formal-reasoning tools, but other applications are admissible as well.
\<close>


chapter \<open>Isabelle/PIDE\<close>

section \<open>PIDE approach (2009)\<close>

text \<open>
  \<^descr>[Prover] supports asynchronous \hilite{document model} natively

  \<^bigskip>

  \<^descr>[Editor] continuously sends source \hilite{edits} and receives markup
  \hilite{reports}

  \<^bigskip>

  \<^descr>[Tools] may \hilite{participate} in document processing and markup

  \<^bigskip>

  \<^descr>[User] constructs document content --- assisted by \\
  \hilite{GUI rendering} of cumulative \hilite{PIDE markup}
\<close>


section \<open>Isabelle/jEdit Prover IDE (2016)\<close>

text \<open>
  \centering
  \includegraphics[height=0.8\textheight]{isabelle-jedit}
\<close>


section \<open>PIDE architecture\<close>

text \<open>
  \begin{center}
  \includegraphics[width=0.8\textwidth]{arch2}
  \end{center}
\<close>

paragraph \<open>Design principles:\<close>
text \<open>
  \<^item> \red{private} protocol for prover connectivity \\
  (asynchronous interaction, parallel evaluation)

  \<^item> \red{public} Scala API \\
  (timeless, stateless, static typing)
\<close>


section \<open>PIDE markup reports\<close>

paragraph \<open>Problem:
  \plainstyle round-trip through several sophisticated syntax layers\<close>

paragraph \<open>Solution:
  \plainstyle execution trace with \hilite{markup reports}\<close>
text \<open>
  \<^bigskip>
  \begin{center}
    \includegraphics[width=0.6\textwidth]{reports}
  \end{center}
  \<^bigskip>
\<close>

paragraph \<open>Example:
  \plainstyle semantic markup for \hilite{domain-specific formal languages}\<close>


chapter \<open>Isabelle document language\<close>

section \<open>Document structure\<close>

paragraph \<open>Markup\<close>
text \<open>
  \<^item> section headings (6 levels like in HTML): \\
    \<^theory_text>\<open>chapter\<close>, \<^theory_text>\<open>section\<close>, \<^theory_text>\<open>subsection\<close>, \dots, \<^theory_text>\<open>subparagraph\<close>
  \<^item> text blocks: \<^theory_text>\<open>text\<close>, \<^theory_text>\<open>txt\<close>, \<^theory_text>\<open>text_raw\<close>
  \<^item> free-form {\LaTeX} macros (\red{rare})
\<close>

paragraph \<open>Markdown\<close>
text \<open>
  \<^item> implicit paragraphs and lists: itemize, enumerate, description
\<close>


section \<open>Document antiquotations\<close>

text \<open>
  \<^descr>[full form:] \red{\<^verbatim>\<open>@{\<close>\<open>name [options] arguments \<dots>\<close>\<^verbatim>\<open>}\<close>}
  \<^descr>[short form:] ~
      \<^enum> cartouche argument: \red{\<^verbatim>\<open>\\<close>\<^verbatim>\<open><^\<close>\<open>name\<close>\<^verbatim>\<open>>\<close>\<open>\<open>argument\<close>\<close>}
      \<^enum> no argument: \red{\<^verbatim>\<open>\\<close>\<^verbatim>\<open><^\<close>\<open>name\<close>\<^verbatim>\<open>>\<close>}
      \<^enum> standard name: \red{\<open>\<open>argument\<close>\<close>}
\<close>

paragraph \<open>Notable examples:\<close>
text \<open>
  \<^item> \<open>cartouche\<close>, \<open>theory_text\<close>: self-presentation of Isar
  \<^item> \<open>bold\<close>, \<open>emph\<close>, \<open>verbatim\<close>, \<open>footnote\<close>: text styles (with proper nesting)
  \<^item> \<open>noindent\<close>, \<open>smallskip\<close>, \<open>medskip\<close>, \<open>bigskip\<close>: spacing
  \<^item> \<open>cite\<close>: formal {Bib\TeX} items
  \<^item> \<open>path\<close>, \<open>file\<close>, \<open>dir\<close>, \<open>url\<close>, \<open>doc\<close>: system resources
\<close>


chapter \<open>Isabelle/ML IDE\<close>

section \<open>Continuous feedback from static analysis and semantic evaluation\<close>

text \<open>
  \<^item> available for years, e.g. Isabelle2014 (August 2014)

  \<^item> corollary of PIDE interaction and Poly/ML compiler integration

    \<^item> static checking (syntax)
      e.g. warnings, errors, inferred types, references to defined items

    \<^item> dynamic evaluation (semantics)
      e.g. toplevel output

  \<open>\<longrightarrow>\<close> Isabelle/PIDE as semantic editor for Isabelle/ML and Standard ML
\<close>

paragraph \<open>Example: inlined ML source\<close>

ML \<open>
  fun factorial 0 = 1
    | factorial n = n * factorial (n - 1)
\<close>

ML \<open>factorial 42\<close>
ML \<open>factorial 10000 div factorial 9999\<close>


paragraph \<open>Example: external ML source\<close>

ML_file "ackermann.ML"

text \<open>
  \<^item> \<^theory_text>\<open>ML_file\<close> better suited for big modules

  \<^item> more editor support (mode \<^verbatim>\<open>isabelle-ml\<close>): keywords, Sidekick, folding
\<close>

paragraph \<open>Example: big ML projects within theory hierarchy\<close>

text \<open>
  \<^file>\<open>~~/src/HOL/HOL.thy\<close>
  \<^file>\<open>~~/src/Tools/quickcheck.ML\<close>
\<close>


paragraph \<open>Example: official Standard ML\<close>

text \<open>See \<^file>\<open>~~/src/Tools/SML/Examples.thy\<close>.\<close>


section \<open>Source-level debugging within the IDE\<close>

text \<open>
  \<^item> new in Poly/ML 5.6, which is bundled with Isabelle2016 (February 2016)

  \<^item> \<^verbatim>\<open>Debugger\<close> dockable (@{action "isabelle-debugger"}):

    \<^item> connect to running ML threads
    \<^item> inspect stack frame with local ML bindings
    \<^item> evaluate ML expressions in run-time context

  See also \<^doc>\<open>jedit\<close> chapter 5 or
  \<^url>\<open>http://sketis.net/2016/ml-debugging-within-the-prover-ide\<close>.
\<close>


paragraph \<open>Example: ML debugger\<close>

context notes [[ML_debugger]]
begin

ML \<open>
  fun factorial 0 = 1
    | factorial n = n * factorial (n - 1)
\<close>

end

ML \<open>factorial 10\<close>


section \<open>IDE support for the Isabelle/Pure bootstrap process\<close>

paragraph \<open>Entry points (independent!)\<close>

text \<open>
  \<^item> \<^file>\<open>~~/src/Pure/ROOT0.ML\<close> -- pre-bootstrap
  \<^item> \<^file>\<open>~~/src/Pure/ROOT.ML\<close> -- main bootstrap
  \<^item> \<^file>\<open>~~/src/Pure/Pure.thy\<close> -- final Pure setup (Isar commands)
  \<^item> \<^file>\<open>~~/src/Pure/ML_Bootstrap.thy\<close> -- theory context for ML bootstrap
\<close>

paragraph \<open>Bootstrap state\<close>

text \<open>
  \<^item> \<^file>\<open>~~/src/Pure/Concurrent/thread_data.ML\<close> -- physical thread data
  \<^item> \<^file>\<open>~~/src/Pure/Concurrent/thread_data_virtual.ML\<close> -- Isabelle context data
\<close>


section \<open>Standalone SML projects\<close>

paragraph \<open>Example: MetiTarski\<close>

text \<open>
  \<^verbatim>\<open>
    hg clone https://bitbucket.org/lcpaulson/metitarski
    hg import -R metitarski --no-commit metitarski-change
  \<close>

  \<^item> original: \<^url>\<open>https://bitbucket.org/lcpaulson/metitarski\<close>

  \<^item> minimal change: \<^file>\<open>metitarski-change\<close>

  \<^item> main directory: \<^dir>\<open>metitarski\<close>

  \<^item> editable project sources: \<^file>\<open>metitarski/src/ROOT.ML\<close>
\<close>


chapter \<open>Conclusions\<close>

section \<open>Conclusions\<close>

text \<open>
  \<^descr>[Self-application] of Isabelle/PIDE to Isabelle/Pure

  \<^bigskip>

  \<^descr>[Revival of Standard ML] as Isabelle/ML (based on Poly/ML)

  \<^bigskip>

  \<^descr>[Demonstration] for other \hilite{Domain-specific Formal Languages}
\<close>


section \<open>Try it yourself!\<close>

paragraph \<open>Current release:
  \plainstyle February 2016\<close>
text \<open>\<^url>\<open>http://isabelle.in.tum.de\<close>\<close>

paragraph \<open>Next release:
  \plainstyle December 2016\<close>
text \<open>\<^url>\<open>http://isabelle.in.tum.de/website-Isabelle2016-1-RC2\<close>\<close>

end
